import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleLinear, max, format, extent } from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { Dropdown } from './Dropdown';

const width = 960;
const menuHeight = 75;
const height = 500 - menuHeight;
const margin = {
  top: 20,
  right: 30,
  bottom: 65,
  left: 90,
};
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 40;

const attributes = [
  { value: 'sepal_length', label: 'Sepal length' },
  { value: 'sepal_width', label: 'Sepal width' },
  { value: 'petal_length', label: 'Petal length' },
  { value: 'petal_width', label: 'Petal width' },
  { value: 'Species', label: 'Species' },
];

const getLabel = (value) => {
  for (let i = 0; i < attributes.length; i++) {
  	if(attributes[i].value == value) {
    	return attributes[i].label;
    }
  }
};

const App = () => {
  const data = useData();

  const initialXAttribute = 'petal_length';
  const [xAttribute, setXAttribute] = useState(
    initialXAttribute
  );
  const xValue = (d) => d[xAttribute];
  const xAxisLabel = getLabel(xAttribute);

  //   const yValue = (d) => d.sepal_width;
  // const yAxisLabel = 'Sepal width';

  const initialYAttribute = 'sepal_width';
  const [yAttribute, setYAttribute] = useState(
    initialYAttribute
  );
  const yValue = (d) => d[yAttribute];
  const yAxisLabel = getLabel(yAttribute);

  if (!data) {
    return <pre>Loading...</pre>;
  }

  console.log(data.columns);

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  const siFormat = format('.2s');
  const xAxisTickFormat = (tickValue) =>
    siFormat(tickValue).replace('G', 'B');

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight]);

  return (
    <>
      <label for="x-select">X:</label>
      <Dropdown
        options={attributes}
        id="x-select"
        selectedValue={xAttribute}
        onSelectedValueChange={setXAttribute}
      />
      <label for="y-select">Y:</label>
      <Dropdown
        options={attributes}
        id="y-select"
        selectedValue={yAttribute}
        onSelectedValueChange={setYAttribute}
      />
      <svg width={width} height={height}>
        <g
          transform={`translate(${margin.left}, ${margin.top})`}
        >
          <AxisBottom
            tickFormat={xAxisTickFormat}
            xScale={xScale}
            innerHeight={innerHeight}
            tickOffset={5}
          />
          <AxisLeft
            yScale={yScale}
            innerWidth={innerWidth}
            tickOffset={5}
          />
          <text
            className="axis-label"
            textAnchor="middle"
            transform={`rotate(-90) translate(${
              -innerHeight / 2
            }, ${-yAxisLabelOffset})`}
          >
            {yAxisLabel}
          </text>
          <text
            className="axis-label"
            x={innerWidth / 2}
            y={innerHeight + xAxisLabelOffset}
            textAnchor={'middle'}
          >
            {xAxisLabel}
          </text>
          <Marks
            data={data}
            yScale={yScale}
            xScale={xScale}
            xValue={xValue}
            yValue={yValue}
            tooltipFormat={xAxisTickFormat}
            circleRadius={7}
          />
        </g>
      </svg>
    </>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
